import { TreeGrid } from '../base';
/**
 * The `Page` module is used to render pager and handle paging action.
 *
 * @hidden
 */
export declare class Page {
    private parent;
    constructor(parent: TreeGrid);
    /**
     * @hidden
     * @returns {void}
     */
    addEventListener(): void;
    /**
     * @hidden
     * @returns {void}
     */
    removeEventListener(): void;
    /**
     * For internal use only - Get the module name.
     *
     * @private
     * @returns {string} Returns Pager module name
     */
    protected getModuleName(): string;
    /**
     * Refreshes the page count, pager information, and external message.
     *
     * @returns {void}
     */
    refresh(): void;
    /**
     * To destroy the pager
     *
     * @returns {void}
     * @hidden
     */
    destroy(): void;
    /**
     * Navigates to the target page according to the given number.
     *
     * @param  {number} pageNo - Defines the page number to navigate.
     * @returns {void}
     */
    goToPage(pageNo: number): void;
    /**
     * Defines the text of the external message.
     *
     * @param  {string} message - Defines the message to update.
     * @returns {void}
     */
    updateExternalMessage(message: string): void;
    /**
     * @param {{action: string, row: HTMLTableRowElement, record: ITreeData, args: RowCollapsedEventArgs}} rowDetails - Expand Collapse details arguments
     * @param {string} rowDetails.action - Expand Collapse action type
     * @param {HTMLTableRowElement} rowDetails.row - Row element
     * @param {ITreeData} rowDetails.record - Row object data
     * @param {RowCollapsedEventArgs} rowDetails.args - Expand Collapse event arguments
     * @hidden
     * @returns {void}
     */
    private collapseExpandPagedchilds;
    private pageRoot;
    private updatePageSize;
    private pageAction;
}
